@@ -512,26 +512,6 @@ static int display(Queenscreen *qs)
   return polys;
 }
 
-static const GLfloat spidermodel[][3] =
-  {
-    {0.48, 0.48, 0.22},
-    {0.48, 0.34, 0.18},
-    {0.34, 0.34, 0.10},
-    {0.34, 0.18, 0.30},
-    {0.18, 0.14, 0.38},
-    {0.14, 0.29, 0.01},
-    {0.29, 0.18, 0.18},
-    {0.18, 0.18, 0.16},
-    {0.18, 0.20, 0.26},
-    {0.20, 0.27, 0.14},
-    {0.27, 0.24, 0.08},
-    {0.24, 0.17, 0.00},
-    {0.17, 0.095, 0.08},
-    {0.095, 0.07, 0.00},
-    {0.07, 0.00, 0.12},
-  };
-
-
 #define EPSILON 0.001
 
 #if 0
@@ -577,6 +557,10 @@ ENTRYPOINT void init_queens(ModeInfo *mi)
   int poly_counts[PIECES];
   wire = MI_IS_WIREFRAME(mi);
 
+# ifdef HAVE_JWZGLES /* #### glPolygonMode other than GL_FILL unimplemented */
+  wire = 0;
+# endif
+
   if(!qss && 
      !(qss = (Queenscreen *) calloc(MI_NUM_SCREENS(mi), sizeof(Queenscreen))))
     return;
